//
// adi_ttcp.h
//
// Contains #defines, struct definitions and data and function declarations
// common to all ports of ADI-TTCP. Sources that #include this file should
// first #include a "config.h" that defines the types BOOLEAN, USHORT,
// SOCKADDR_IN, etc.
//

#ifndef _ADI_TTCP_H
#define _ADI_TTCP_H

#define ADI_TTCP_VERSION "1.0.1"

#ifndef CONTROL_PORT
#define CONTROL_PORT 6000  // default listen port for control stream connections
#endif

#ifndef LISTEN_PORT_BASE
#define LISTEN_PORT_BASE 5000  // default listen ports for relayed control
#define LISTEN_PORT_MAX  5999  // stream connections
#endif

#define UDP_GUARD_BUFFER_LENGTH  4
#define PORT_UNSET 0

typedef struct _MSG_BLOCK
{
    PCHAR	m_msgs;              // for holding all of transfer's messages
    size_t	m_msgs_capacity;     // the remaining capacity in m_msgs
} MSG_BLOCK, *PMSG_BLOCK;

typedef
struct _CMD_BLOCK
{
    //
    // Test Command/Configuration Data
    //
    BOOLEAN     m_bTransmit;

    USHORT      m_Protocol;

    SOCKADDR_IN m_RemoteAddr;        // Host Byte Order
    int         m_nRemoteAddrLen;

    SOCKADDR_IN m_LocalAddress;      // Host Byte Order

    USHORT      m_Port;              // Host Byte Order

    BOOLEAN     m_bOptContinuous;
    int         m_nOptWriteDelay;    // microseconds of delay before each write

    BOOLEAN     m_bTouchRecvData;

    int         m_nNumBuffersToSend; // number of buffers to send

    int         m_nBufferSize;       // length of buffer
    int         m_nBufOffset;        // align buffer to this
    int         m_nBufAlign;         // modulo this

    BOOLEAN     m_bUseSockOptBufSize;
    int         m_nSockOptBufSize;   // socket buffer size to use

    int         m_nSockOptNoDelay;   // set TCP_NODELAY socket option

    BOOLEAN     m_bSockOptDebug;     // TRUE = set SO_DEBUG socket option

    int         m_TokenRate;         // Desired Data Transmit Rate, Bytes/Sec
    TIME	m_period;            // Maximum duration of transfer (millisec)

    SOCKET      m_ExtraSocket;

    MSG_BLOCK	m_msgBlk;

    char	m_fmt;

} CMD_BLOCK, *PCMD_BLOCK;

struct _TEST_BLOCK;
typedef struct _TEST_BLOCK* PTEST_BLOCK;

typedef
struct _TEST_BLOCK
{
    //
    // Command/Configuration Data
    //
    CMD_BLOCK   m_Cmd;               // must be first member of TEST_BLOCK
    //
    // Instance Data
    //
    PCHAR       m_pBufBase;          // ptr to dynamic buffer (base)
    PCHAR       m_pBuf;              // ptr to dynamic buffer (aligned)

    SOCKET      m_Socket_fd;         // fd of network send/receive socket
    //
    // Statistics
    //
    TIME        m_tStart;
    TIME        m_tFinish;

    TARGET_SPECIFIC_FIELDS           // defined in each "config.h"

    ULONG       m_numCalls;          // # of I/O system calls
    ULONGLONG   m_nbytes;            // bytes on net
    DWORD       m_ByteCredits;       // For Data Rate Pacing
    //
    // Housekeeping
    //
    int         m_id;                // this TRANSFER's unique identifier
    PTEST_BLOCK m_chain;             // for linking all TRANSFER requests

} TEST_BLOCK;

struct _RELAY_BLOCK;
typedef struct _RELAY_BLOCK* PRELAY_BLOCK;

typedef struct _RELAY_BLOCK
{
    //
    // Information about one RELAY request
    //
    SOCKET	m_listen_socket;     // where controller should connect to
    USHORT	m_listen_port;

    SOCKET	m_control_socket;    // connection accepted from controller

    SOCKET	m_remote_socket;     // remote ADI-TTCP's control stream

    PRELAY_BLOCK m_chain;            // for linking all RELAY requests
    MSG_BLOCK	m_msgBlk;

} RELAY_BLOCK;

extern SOCKET control_fd;
extern USHORT control_port;

// -----------------------------------------------------------------------------
//
// functions that a port of ADI-TTCP must provide
//
// print a message on machine running ADI-TTCP (if possible)
extern void TTCP_ConsoleMsg(const char* fmt, ...);

// return numeric error code for last TCP/IP library error
extern DWORD TTCP_GetLastError(void);

// run all transfers concurrently; return when all completed
extern int TTCP_RunTransfers(PTEST_BLOCK tfr_list, int (despatch)(void*));

// terminate the running thread because of an error
extern void TTCP_TerminateThread(PTEST_BLOCK ptfr);

// create and run a thread to perform a RELAY command; return immediately
extern void TTCP_CreateRelay(PRELAY_BLOCK prly, void (relay)(void*));

// return relative time measured in milliseconds
extern TIME TTCP_GetMillisecCount(void);

// sleep for at least the requested number of milliseconds
extern void TTCP_SleepMillisec(DWORD millisec);

// delay for at least the requested number of microseconds
extern void TTCP_DelayMicrosec(DWORD microsec);

// perform any target specific start-of-transfer actions
extern void TTCP_TransferStarting(PTEST_BLOCK ptfr);

// perform any target specific end-of-transfer actions
extern void TTCP_TransferStopping(PTEST_BLOCK ptfr);

// return CPU idle time (in seconds) for this transfer, if available
extern double TTCP_CPUIdleTime(PTEST_BLOCK ptfr);

// -----------------------------------------------------------------------------
//
// functions that ADI-TTCP common code provides
//
// execute the ADI-TTCP main control loop
extern int TTCP_MainLoop(void);

// call appropriate function to execute transfer
extern int TTCP_DispatchTransfer(void* arg);

// execute a RELAY command
extern void TTCP_RunRelay(void* arg);

// add a message to a TRANSFER or RELAY command
extern char* TTCP_AddMsg(PMSG_BLOCK pmsg, char* fmt, ...);

// return character representation of IP address and port
extern char* TTCP_GetAddr(SOCKADDR_IN* sa, char* buf);

#endif /* _ADI_TTCP_H */
